@@ -4,14 +4,19 @@ use std::convert::TryFrom;
 use std::fmt;
 use std::iter::FromIterator;
 
+use async_hash::{hash_try_stream, Hash};
 use async_trait::async_trait;
 use destream::{de, en};
 use futures::future::{join_all, try_join_all, TryFutureExt};
 use log::debug;
 use safecast::{TryCastFrom, TryCastInto};
+use sha2::digest::generic_array::GenericArray;
+use sha2::digest::{Digest, Output};
+use sha2::Sha256;
 
-use tc_btree::{BTreeType, Column};
+use tc_btree::{BTreeInstance, BTreeType, Column};
 use tc_error::*;
+use tc_table::TableStream;
 #[cfg(feature = "tensor")]
 use tc_tensor::TensorPersist;
 use tc_transact::fs::{Dir, Persist, Restore, Store};
@@ -24,7 +29,8 @@ use crate::collection::{
 };
 #[cfg(feature = "tensor")]
 use crate::collection::{
-    DenseTensor, DenseTensorFile, SparseTable, SparseTensor, Tensor, TensorType,
+    DenseAccess, DenseTensor, DenseTensorFile, SparseAccess, SparseTable, SparseTensor, Tensor,
+    TensorType,
 };
 use crate::fs;
 use crate::scalar::{OpRef, Scalar, TCRef};
@@ -42,7 +48,6 @@ mod sync;
 
 const BLOCK_SIZE: usize = 1_000_000;
 const CHAIN: Label = label("chain");
-const NULL_HASH: Vec<u8> = vec![];
 const PREFIX: PathLabel = path_label(&["state", "chain"]);
 
 const SUBJECT: Label = label("subject");
@@ -51,12 +56,14 @@ const SUBJECT: Label = label("subject");
 #[derive(Clone)]
 pub enum Schema {
     BTree(tc_btree::RowSchema),
+    Map(Map<Schema>),
     Table(tc_table::TableSchema),
+    Tuple(Tuple<Schema>),
+
     #[cfg(feature = "tensor")]
     Dense(tc_tensor::Schema),
     #[cfg(feature = "tensor")]
     Sparse(tc_tensor::Schema),
-    Tuple(Tuple<Schema>),
 }
 
 impl Schema {
@@ -110,11 +117,19 @@ impl Schema {
                 },
                 other => Err(TCError::bad_request("invalid Chain schema", other)),
             },
+
+            Scalar::Map(map) => map
+                .into_iter()
+                .map(|(name, scalar)| Schema::from_scalar(scalar).map(|schema| (name, schema)))
+                .collect::<TCResult<Map<Schema>>>()
+                .map(Schema::Map),
+
             Scalar::Tuple(tuple) => tuple
                 .into_iter()
                 .map(|scalar| Schema::from_scalar(scalar))
                 .collect::<TCResult<Tuple<Schema>>>()
                 .map(Schema::Tuple),
+
             other => Err(TCError::bad_request("invalid Chain schema", other)),
         }
     }
@@ -140,18 +155,20 @@ impl<'en> en::IntoStream<'en> for Schema {
                 map.encode_entry(BTreeType::default().path(), (schema,))?;
                 map.end()
             }
+            Self::Map(map) => map.into_stream(encoder),
             Self::Table(schema) => {
                 let mut map = encoder.encode_map(Some(1))?;
                 map.encode_entry(TableType::default().path(), (schema,))?;
                 map.end()
             }
+            Self::Tuple(tuple) => tuple.into_stream(encoder),
+
             #[cfg(feature = "tensor")]
             Self::Dense(schema) | Self::Sparse(schema) => {
                 let mut map = encoder.encode_map(Some(1))?;
                 map.encode_entry(TensorType::Dense.path(), (schema,))?;
                 map.end()
             }
-            Self::Tuple(tuple) => tuple.into_stream(encoder),
         }
     }
 }
@@ -160,12 +177,14 @@ impl fmt::Display for Schema {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Self::BTree(schema) => write!(f, "{}", Tuple::<&Column>::from_iter(schema)),
+            Self::Map(schema) => fmt::Display::fmt(schema, f),
+            Self::Table(schema) => fmt::Display::fmt(schema, f),
+            Self::Tuple(schema) => fmt::Display::fmt(schema, f),
+
             #[cfg(feature = "tensor")]
             Self::Dense(schema) => fmt::Display::fmt(schema, f),
             #[cfg(feature = "tensor")]
             Self::Sparse(schema) => fmt::Display::fmt(schema, f),
-            Self::Table(schema) => fmt::Display::fmt(schema, f),
-            Self::Tuple(tuple) => fmt::Display::fmt(tuple, f),
         }
     }
 }
@@ -174,12 +193,14 @@ impl fmt::Display for Schema {
 #[derive(Clone)]
 pub enum Subject {
     BTree(BTreeFile),
+    Map(Map<Subject>),
     Table(TableIndex),
+    Tuple(Tuple<Subject>),
+
     #[cfg(feature = "tensor")]
     Dense(DenseTensor<DenseTensorFile>),
     #[cfg(feature = "tensor")]
     Sparse(SparseTensor<SparseTable>),
-    Tuple(Tuple<Subject>),
 }
 
 impl Subject {
@@ -225,6 +246,17 @@ impl Subject {
                     .map_ok(Self::Tuple)
                     .await
                 }
+                Schema::Map(schema) => {
+                    try_join_all(schema.into_iter().map(|(name, schema)| async move {
+                        let dir = dir.create_dir(txn_id, name.clone()).await?;
+                        Self::create(schema, &dir, txn_id)
+                            .map_ok(|subject| (name, subject))
+                            .await
+                    }))
+                    .map_ok(|schemata| schemata.into_iter().collect())
+                    .map_ok(Self::Map)
+                    .await
+                }
                 Schema::BTree(schema) => {
                     let file = dir
                         .create_file(txn_id, SUBJECT.into(), BTreeType::default())
@@ -248,6 +280,24 @@ impl Subject {
                         Self::create(Schema::BTree(schema), dir, *txn.id()).await
                     }
                 }
+                Schema::Map(schema) => {
+                    let txn_id = *txn.id();
+                    try_join_all(schema.into_iter().map(|(name, schema)| async move {
+                        if let Some(dir) = dir.get_dir(txn_id, &name).await? {
+                            Self::load(txn, schema, &dir)
+                                .map_ok(|subject| (name, subject))
+                                .await
+                        } else {
+                            let dir = dir.create_dir(txn_id, name.clone()).await?;
+                            Self::create(schema, &dir, txn_id)
+                                .map_ok(|subject| (name, subject))
+                                .await
+                        }
+                    }))
+                    .map_ok(|subjects| subjects.into_iter().collect())
+                    .map_ok(Self::Map)
+                    .await
+                }
                 Schema::Table(schema) => {
                     if dir.is_empty(*txn.id()).await? {
                         Self::create(Schema::Table(schema), dir, *txn.id()).await
@@ -257,26 +307,6 @@ impl Subject {
                             .await
                     }
                 }
-                #[cfg(feature = "tensor")]
-                Schema::Dense(schema) => {
-                    if let Some(file) = dir.get_file(*txn.id(), &SUBJECT.into()).await? {
-                        DenseTensor::load(txn, schema, file)
-                            .map_ok(Self::Dense)
-                            .await
-                    } else {
-                        Self::create(Schema::Dense(schema), dir, *txn.id()).await
-                    }
-                }
-                #[cfg(feature = "tensor")]
-                Schema::Sparse(schema) => {
-                    if let Some(dir) = dir.get_dir(*txn.id(), &SUBJECT.into()).await? {
-                        SparseTensor::load(txn, schema, dir)
-                            .map_ok(Self::Sparse)
-                            .await
-                    } else {
-                        Self::create(Schema::Sparse(schema), dir, *txn.id()).await
-                    }
-                }
                 Schema::Tuple(schema) => {
                     let txn_id = *txn.id();
                     try_join_all(
@@ -296,6 +326,27 @@ impl Subject {
                     .map_ok(Self::Tuple)
                     .await
                 }
+
+                #[cfg(feature = "tensor")]
+                Schema::Dense(schema) => {
+                    if let Some(file) = dir.get_file(*txn.id(), &SUBJECT.into()).await? {
+                        DenseTensor::load(txn, schema, file)
+                            .map_ok(Self::Dense)
+                            .await
+                    } else {
+                        Self::create(Schema::Dense(schema), dir, *txn.id()).await
+                    }
+                }
+                #[cfg(feature = "tensor")]
+                Schema::Sparse(schema) => {
+                    if let Some(dir) = dir.get_dir(*txn.id(), &SUBJECT.into()).await? {
+                        SparseTensor::load(txn, schema, dir)
+                            .map_ok(Self::Sparse)
+                            .await
+                    } else {
+                        Self::create(Schema::Sparse(schema), dir, *txn.id()).await
+                    }
+                }
             }
         })
     }
@@ -303,6 +354,7 @@ impl Subject {
     fn restore<'a>(&'a self, txn: &'a Txn, backup: State) -> TCBoxTryFuture<()> {
         Box::pin(async move {
             let txn_id = *txn.id();
+
             match self {
                 Self::BTree(btree) => match backup {
                     State::Collection(Collection::BTree(BTree::File(backup))) => {
@@ -310,6 +362,64 @@ impl Subject {
                     }
                     other => Err(TCError::bad_request("cannot restore a BTree from", other)),
                 },
+                Self::Map(map) => match backup {
+                    State::Map(mut backups) => {
+                        let backups = map
+                            .iter()
+                            .map(|(name, subject)| {
+                                backups
+                                    .remove(name)
+                                    .ok_or_else(|| {
+                                        TCError::bad_request(
+                                            "backup not found for Chain subject",
+                                            name,
+                                        )
+                                    })
+                                    .map(|backup| (subject, backup))
+                            })
+                            .collect::<TCResult<Vec<(&Subject, State)>>>()?;
+
+                        let restores = backups
+                            .into_iter()
+                            .map(|(subject, backup)| subject.restore(txn, backup));
+
+                        try_join_all(restores).await?;
+                        Ok(())
+                    }
+                    backup => Err(TCError::unsupported(format!(
+                        "invalid backup for schema {}: {}",
+                        map, backup
+                    ))),
+                },
+                Self::Table(table) => match backup {
+                    State::Collection(Collection::Table(Table::Table(backup))) => {
+                        table.restore(&backup, txn_id).await
+                    }
+                    other => Err(TCError::bad_request("cannot restore a Table from", other)),
+                },
+                Self::Tuple(tuple) => match backup {
+                    State::Tuple(backup) if backup.len() == tuple.len() => {
+                        let restores =
+                            tuple
+                                .iter()
+                                .zip(backup)
+                                .map(|(subject, backup)| async move {
+                                    subject.restore(txn, backup).await
+                                });
+
+                        try_join_all(restores).await?;
+                        Ok(())
+                    }
+                    State::Tuple(_) => Err(TCError::bad_request(
+                        "backup has the wrong number of subjects for schema",
+                        tuple,
+                    )),
+                    backup => Err(TCError::unsupported(format!(
+                        "invalid backup for schema {}: {}",
+                        tuple, backup
+                    ))),
+                },
+
                 #[cfg(feature = "tensor")]
                 Self::Dense(tensor) => match backup {
                     State::Collection(Collection::Tensor(Tensor::Dense(backup))) => {
@@ -340,34 +450,53 @@ impl Subject {
                         other,
                     )),
                 },
-                Self::Table(table) => match backup {
-                    State::Collection(Collection::Table(Table::Table(backup))) => {
-                        table.restore(&backup, txn_id).await
-                    }
-                    other => Err(TCError::bad_request("cannot restore a Table from", other)),
-                },
-                Self::Tuple(tuple) => match backup {
-                    State::Tuple(backup) if backup.len() == tuple.len() => {
-                        let restores =
-                            tuple
-                                .iter()
-                                .zip(backup)
-                                .map(|(subject, backup)| async move {
-                                    subject.restore(txn, backup).await
-                                });
+            }
+        })
+    }
 
-                        try_join_all(restores).await?;
-                        Ok(())
+    pub fn hash<'a>(self, txn: Txn) -> TCBoxTryFuture<'a, Output<Sha256>> {
+        Box::pin(async move {
+            // TODO: should this be consolidated with Collection::hash?
+            match self {
+                Self::BTree(btree) => {
+                    let keys = btree.keys(*txn.id()).await?;
+                    hash_try_stream::<Sha256, _, _, _>(keys).await
+                }
+                Self::Map(map) => {
+                    let mut hasher = Sha256::default();
+                    for (id, subject) in map {
+                        let subject = subject.hash(txn.clone()).await?;
+
+                        let mut inner_hasher = Sha256::default();
+                        inner_hasher.update(&Hash::<Sha256>::hash(id));
+                        inner_hasher.update(&subject);
+                        hasher.update(&inner_hasher.finalize());
                     }
-                    State::Tuple(_) => Err(TCError::bad_request(
-                        "backup has the wrong number of subjects for schema",
-                        tuple,
-                    )),
-                    backup => Err(TCError::unsupported(format!(
-                        "invalid backup for schema {}: {}",
-                        tuple, backup
-                    ))),
-                },
+
+                    Ok(hasher.finalize())
+                }
+                Self::Table(table) => {
+                    let rows = table.rows(*txn.id()).await?;
+                    hash_try_stream::<Sha256, _, _, _>(rows).await
+                }
+                Self::Tuple(tuple) => {
+                    let mut hasher = Sha256::default();
+                    for subject in tuple {
+                        let subject = subject.hash(txn.clone()).await?;
+                        hasher.update(&subject);
+                    }
+                    Ok(hasher.finalize())
+                }
+                #[cfg(feature = "tensor")]
+                Self::Dense(dense) => {
+                    let elements = dense.into_inner().value_stream(txn).await?;
+                    hash_try_stream::<Sha256, _, _, _>(elements).await
+                }
+                #[cfg(feature = "tensor")]
+                Self::Sparse(sparse) => {
+                    let filled = sparse.into_inner().filled(txn).await?;
+                    hash_try_stream::<Sha256, _, _, _>(filled).await
+                }
             }
         })
     }
@@ -379,12 +508,13 @@ impl Instance for Subject {
     fn class(&self) -> Self::Class {
         match self {
             Self::BTree(btree) => CollectionType::BTree(btree.class()).into(),
+            Self::Map(_) => StateType::Map,
+            Self::Table(table) => CollectionType::Table(table.class()).into(),
+            Self::Tuple(_) => StateType::Tuple,
             #[cfg(feature = "tensor")]
             Self::Dense(dense) => CollectionType::Tensor(dense.class()).into(),
             #[cfg(feature = "tensor")]
             Self::Sparse(sparse) => CollectionType::Tensor(sparse.class()).into(),
-            Self::Table(table) => CollectionType::Table(table.class()).into(),
-            Self::Tuple(_) => StateType::Tuple,
         }
     }
 }
@@ -396,11 +526,14 @@ impl Transact for Subject {
 
         match self {
             Self::BTree(btree) => btree.commit(txn_id).await,
+            Self::Map(map) => {
+                join_all(
+                    map.iter()
+                        .map(|(_, subject)| async move { subject.commit(txn_id).await }),
+                )
+                .await;
+            }
             Self::Table(table) => table.commit(txn_id).await,
-            #[cfg(feature = "tensor")]
-            Self::Dense(tensor) => tensor.commit(txn_id).await,
-            #[cfg(feature = "tensor")]
-            Self::Sparse(tensor) => tensor.commit(txn_id).await,
             Self::Tuple(tuple) => {
                 join_all(
                     tuple
@@ -409,6 +542,10 @@ impl Transact for Subject {
                 )
                 .await;
             }
+            #[cfg(feature = "tensor")]
+            Self::Dense(tensor) => tensor.commit(txn_id).await,
+            #[cfg(feature = "tensor")]
+            Self::Sparse(tensor) => tensor.commit(txn_id).await,
         }
     }
 
@@ -417,17 +554,23 @@ impl Transact for Subject {
 
         match self {
             Self::BTree(btree) => btree.finalize(txn_id).await,
+            Self::Map(map) => {
+                join_all(map.iter().map(|(_, subject)| async move {
+                    subject.finalize(txn_id).await;
+                }))
+                .await;
+            }
             Self::Table(table) => table.finalize(txn_id).await,
-            #[cfg(feature = "tensor")]
-            Self::Dense(tensor) => tensor.finalize(txn_id).await,
-            #[cfg(feature = "tensor")]
-            Self::Sparse(tensor) => tensor.finalize(txn_id).await,
             Self::Tuple(tuple) => {
                 join_all(tuple.iter().map(|subject| async move {
                     subject.finalize(txn_id).await;
                 }))
                 .await;
             }
+            #[cfg(feature = "tensor")]
+            Self::Dense(tensor) => tensor.finalize(txn_id).await,
+            #[cfg(feature = "tensor")]
+            Self::Sparse(tensor) => tensor.finalize(txn_id).await,
         }
     }
 }
@@ -464,6 +607,14 @@ fn from_state<E: de::Error>(state: State) -> Result<Subject, E> {
             },
             other => Err(de::Error::invalid_type(other, ERR_INVALID)),
         },
+        State::Map(map) => {
+            let subject = map
+                .into_iter()
+                .map(|(name, state)| from_state(state).map(|subject| (name, subject)))
+                .collect::<Result<Map<Subject>, E>>()?;
+
+            Ok(Subject::Map(subject))
+        }
         State::Tuple(tuple) => {
             let subject = tuple
                 .into_iter()
@@ -484,11 +635,16 @@ impl<'en> IntoView<'en, fs::Dir> for Subject {
     async fn into_view(self, txn: Self::Txn) -> TCResult<Self::View> {
         match self {
             Self::BTree(btree) => State::from(BTree::File(btree)).into_view(txn).await,
+            Self::Map(map) => {
+                let views = map.into_iter().map(|(name, subject)| {
+                    let txn = txn.clone();
+                    async move { subject.into_view(txn).map_ok(|view| (name, view)).await }
+                });
+
+                let views = try_join_all(views).await?;
+                Ok(StateView::Map(views.into_iter().collect()))
+            }
             Self::Table(table) => State::from(Table::Table(table)).into_view(txn).await,
-            #[cfg(feature = "tensor")]
-            Self::Dense(tensor) => State::from(Tensor::from(tensor)).into_view(txn).await,
-            #[cfg(feature = "tensor")]
-            Self::Sparse(tensor) => State::from(Tensor::from(tensor)).into_view(txn).await,
             Self::Tuple(tuple) => {
                 let views = tuple
                     .into_iter()
@@ -496,6 +652,11 @@ impl<'en> IntoView<'en, fs::Dir> for Subject {
 
                 try_join_all(views).map_ok(StateView::Tuple).await
             }
+
+            #[cfg(feature = "tensor")]
+            Self::Dense(tensor) => State::from(Tensor::from(tensor)).into_view(txn).await,
+            #[cfg(feature = "tensor")]
+            Self::Sparse(tensor) => State::from(Tensor::from(tensor)).into_view(txn).await,
         }
     }
 }
@@ -504,12 +665,18 @@ impl From<Subject> for State {
     fn from(subject: Subject) -> Self {
         match subject {
             Subject::BTree(btree) => State::Collection(btree.into()),
+            Subject::Map(map) => State::Map(
+                map.into_iter()
+                    .map(|(name, subject)| (name, State::from(subject)))
+                    .collect(),
+            ),
+            Subject::Table(table) => State::Collection(table.into()),
+            Subject::Tuple(tuple) => State::Tuple(tuple.into_iter().map(State::from).collect()),
+
             #[cfg(feature = "tensor")]
             Subject::Dense(dense) => State::Collection(dense.into()),
             #[cfg(feature = "tensor")]
             Subject::Sparse(sparse) => State::Collection(sparse.into()),
-            Subject::Table(table) => State::Collection(table.into()),
-            Subject::Tuple(tuple) => State::Tuple(tuple.into_iter().map(State::from).collect()),
         }
     }
 }
@@ -518,12 +685,14 @@ impl fmt::Display for Subject {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
             Self::BTree(btree) => write!(f, "chain Subject, {}", btree.class()),
+            Self::Map(map) => fmt::Display::fmt(map, f),
             Self::Table(table) => write!(f, "chain Subject, {}", table.class()),
+            Self::Tuple(tuple) => fmt::Display::fmt(tuple, f),
+
             #[cfg(feature = "tensor")]
             Self::Dense(_) => write!(f, "chain Subject, {}", TensorType::Dense),
             #[cfg(feature = "tensor")]
             Self::Sparse(_) => write!(f, "chain Subject, {}", TensorType::Sparse),
-            Self::Tuple(tuple) => fmt::Display::fmt(tuple, f),
         }
     }
 }
@@ -543,6 +712,10 @@ pub trait ChainInstance {
         value: State,
     ) -> TCResult<()>;
 
+    /// Return the latest hash of this `Chain`.
+    async fn hash(self, txn: Txn) -> TCResult<Output<Sha256>>;
+
+    /// Return the `TxnId` of the last commit to this `Chain`, if there is one.
     async fn last_commit(&self, txn_id: TxnId) -> TCResult<Option<TxnId>>;
 
     /// Borrow the [`Subject`] of this [`Chain`] immutably.
@@ -551,6 +724,7 @@ pub trait ChainInstance {
     /// Replicate this [`Chain`] from the [`Chain`] at the given [`Link`].
     async fn replicate(&self, txn: &Txn, source: Link) -> TCResult<()>;
 
+    /// Write the mutation ops in the current transaction to the write-ahead log.
     async fn write_ahead(&self, txn_id: &TxnId);
 }
 
@@ -647,6 +821,13 @@ impl ChainInstance for Chain {
         }
     }
 
+    async fn hash(self, txn: Txn) -> TCResult<Output<Sha256>> {
+        match self {
+            Self::Block(chain) => chain.hash(txn).await,
+            Self::Sync(chain) => chain.hash(txn).await,
+        }
+    }
+
     async fn last_commit(&self, txn_id: TxnId) -> TCResult<Option<TxnId>> {
         match self {
             Self::Block(chain) => chain.last_commit(txn_id).await,
@@ -824,3 +1005,8 @@ impl de::Visitor for ChainVisitor {
         self.visit_map_value(class, &mut map).await
     }
 }
+
+#[inline]
+fn null_hash() -> Output<Sha256> {
+    GenericArray::default()
+}
