@@ -12,7 +12,6 @@ pub use file::*;
 
 mod block;
 mod dir;
-#[allow(unused)]
 mod file;
 
 const VERSION: Label = label(".version");
@@ -22,13 +21,13 @@ fn file_ext(path: &'_ Path) -> Option<&'_ str> {
     path.extension().and_then(|ext| ext.to_str())
 }
 
-pub fn io_err(err: io::Error) -> TCError {
+pub(crate) fn io_err(err: io::Error) -> TCError {
     match err.kind() {
         io::ErrorKind::NotFound => TCError::not_found(err),
         io::ErrorKind::PermissionDenied => TCError::internal(format!(
             "TinyChain does not have permission to access the host filesystem: {}",
             err
         )),
-        _ => TCError::internal(format!("host filesystem error: {}", err)),
+        kind => TCError::internal(format!("host filesystem error: {:?}: {}", kind, err)),
     }
 }
