@@ -51,7 +51,7 @@ fn duration(flag: &str) -> TCResult<Duration> {
 struct Config {
     #[structopt(
         long = "address",
-        default_value = "127.0.0.1",
+        default_value = "0.0.0.0",
         about = "The IP address to bind"
     )]
     pub address: IpAddr,
@@ -122,7 +122,7 @@ async fn main() -> Result<(), TokioError> {
         return Err(TCError::bad_request("the minimum cache size is", MIN_CACHE_SIZE).into());
     }
 
-    let cache = freqfs::Cache::new(config.cache_size as usize, Duration::from_secs(1));
+    let cache = freqfs::Cache::new(config.cache_size as usize, Duration::from_secs(1), None);
     let workspace = cache.clone().load(config.workspace).await?;
     let txn_id = TxnId::new(Gateway::time());
 
