@@ -1,15 +1,20 @@
 //! User-defined class implementation.
 
+use async_hash::Hash;
 use std::fmt;
+use std::ops::Deref;
 
 use async_trait::async_trait;
 use destream::{de, en};
-use safecast::CastFrom;
+use safecast::{CastFrom, TryCastFrom};
+use sha2::digest::Output;
+use sha2::Digest;
 
 use tc_value::{Link, Value};
-use tcgeneric::{path_label, Id, Map, PathLabel, TCPathBuf};
+use tcgeneric::{path_label, Id, Map, NativeClass, PathLabel, TCPathBuf};
 
 use crate::scalar::*;
+use crate::state::StateType;
 
 use super::ObjectType;
 
@@ -48,6 +53,22 @@ impl InstanceClass {
     }
 }
 
+impl<D: Digest> Hash<D> for InstanceClass {
+    fn hash(self) -> Output<D> {
+        Hash::<D>::hash(&self)
+    }
+}
+
+impl<'a, D: Digest> Hash<D> for &'a InstanceClass {
+    fn hash(self) -> Output<D> {
+        if let Some(link) = &self.extends {
+            Hash::<D>::hash((link, self.proto.deref()))
+        } else {
+            Hash::<D>::hash(self.proto.deref())
+        }
+    }
+}
+
 impl tcgeneric::Class for InstanceClass {}
 
 impl tcgeneric::Instance for InstanceClass {
@@ -58,6 +79,41 @@ impl tcgeneric::Instance for InstanceClass {
     }
 }
 
+impl From<StateType> for InstanceClass {
+    fn from(st: StateType) -> Self {
+        Self {
+            extends: Some(st.path().into()),
+            proto: Map::default(),
+        }
+    }
+}
+
+impl TryCastFrom<InstanceClass> for StateType {
+    fn can_cast_from(class: &InstanceClass) -> bool {
+        if class.proto.is_empty() {
+            if let Some(classpath) = &class.extends {
+                if classpath.host().is_none() {
+                    return StateType::from_path(classpath.path()).is_some();
+                }
+            }
+        }
+
+        false
+    }
+
+    fn opt_cast_from(class: InstanceClass) -> Option<Self> {
+        if class.proto.is_empty() {
+            if let Some(classpath) = &class.extends {
+                if classpath.host().is_none() {
+                    return StateType::from_path(classpath.path());
+                }
+            }
+        }
+
+        None
+    }
+}
+
 impl CastFrom<InstanceClass> for Scalar {
     fn cast_from(class: InstanceClass) -> Self {
         Self::Map(class.proto)
