@@ -4,10 +4,13 @@ use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::ops::Deref;
 
+use async_hash::Hash;
 use async_trait::async_trait;
 use destream::{en, EncodeMap};
 use log::debug;
 use safecast::TryCastFrom;
+use sha2::digest::{Digest, Output};
+use sha2::Sha256;
 
 use tc_error::*;
 use tc_transact::IntoView;
@@ -87,6 +90,17 @@ impl<T: tcgeneric::Instance> Deref for InstanceExt<T> {
     }
 }
 
+impl InstanceExt<State> {
+    pub async fn hash(self, txn: Txn) -> TCResult<Output<Sha256>> {
+        let parent = self.parent.hash(txn).await?;
+
+        let mut hasher = Sha256::default();
+        hasher.update(Hash::<Sha256>::hash(self.class));
+        hasher.update(&parent);
+        Ok(hasher.finalize())
+    }
+}
+
 #[async_trait]
 impl<'en> IntoView<'en, Dir> for InstanceExt<State> {
     type Txn = Txn;
