@@ -3,7 +3,7 @@ use tcgeneric::PathSegment;
 use crate::object::{InstanceClass, InstanceExt, Object, ObjectType};
 use crate::state::State;
 
-use super::{GetHandler, Handler, Route};
+use super::{AttributeHandler, GetHandler, Handler, Route, COPY};
 
 mod instance;
 
@@ -25,9 +25,8 @@ impl<'a> Handler<'a> for ClassHandler<'a> {
         Some(Box::new(|_txn, key| {
             Box::pin(async move {
                 let parent = State::from(key);
-                Ok(State::Object(
-                    InstanceExt::new(parent, self.class.clone()).into(),
-                ))
+                let instance = InstanceExt::new(parent, self.class.clone());
+                Ok(State::Object(instance.into()))
             })
         }))
     }
@@ -35,6 +34,12 @@ impl<'a> Handler<'a> for ClassHandler<'a> {
 
 impl Route for InstanceClass {
     fn route<'a>(&'a self, path: &'a [PathSegment]) -> Option<Box<dyn Handler<'a> + 'a>> {
+        if path == &COPY[..] {
+            return Some(Box::new(AttributeHandler::from(Object::Class(
+                self.clone(),
+            ))));
+        }
+
         if path.is_empty() {
             Some(Box::new(ClassHandler { class: self }))
         } else if let Some(attribute) = self.proto().get(&path[0]) {
