@@ -1,5 +1,6 @@
 //! Immutable values which always reside entirely in memory
 
+use async_hash::Hash;
 use std::collections::{BTreeMap, HashSet};
 use std::convert::TryFrom;
 use std::fmt;
@@ -12,10 +13,12 @@ use destream::{de, en};
 use futures::future::TryFutureExt;
 use futures::stream::{self, StreamExt, TryStreamExt};
 use log::{debug, warn};
-use safecast::{Match, TryCastFrom, TryCastInto};
+use safecast::{CastInto, Match, TryCastFrom, TryCastInto};
+use sha2::digest::Output;
+use sha2::Digest;
 
 use tc_error::*;
-use tc_value::{Link, Number, Range, TCString, Value, ValueType};
+use tc_value::{Float, Link, Number, Range, TCString, Value, ValueType};
 use tcgeneric::*;
 
 use crate::closure::Closure;
@@ -128,6 +131,12 @@ impl ClusterRef {
     }
 }
 
+impl<'a, D: Digest> Hash<D> for &'a ClusterRef {
+    fn hash(self) -> Output<D> {
+        Hash::<D>::hash(&self.0)
+    }
+}
+
 impl From<TCPathBuf> for ClusterRef {
     fn from(path: TCPathBuf) -> Self {
         Self(path)
@@ -230,13 +239,6 @@ impl Scalar {
                     .map(Box::new)
                     .map(Scalar::Ref),
 
-                RT::Before => self
-                    .opt_cast_into()
-                    .map(Box::new)
-                    .map(TCRef::Before)
-                    .map(Box::new)
-                    .map(Scalar::Ref),
-
                 RT::Case => self
                     .opt_cast_into()
                     .map(Box::new)
@@ -257,13 +259,6 @@ impl Scalar {
                     .map(Box::new)
                     .map(Scalar::Ref),
 
-                RT::New => self
-                    .opt_cast_into()
-                    .map(Box::new)
-                    .map(TCRef::New)
-                    .map(Box::new)
-                    .map(Scalar::Ref),
-
                 RT::Op(ort) => {
                     debug!("cast into op ref from {}", self);
 
@@ -442,7 +437,18 @@ impl Refer for Scalar {
 
                 Self::Map(map)
             }
-            Self::Op(op_def) => Self::Op(op_def.reference_self(path)),
+            Self::Op(op_def) => {
+                let before = op_def.clone();
+                let after = op_def.reference_self(path);
+                if before == after {
+                    Self::Op(after)
+                } else {
+                    Self::Ref(Box::new(TCRef::With(Box::new(With::new(
+                        vec![SELF.into()].into(),
+                        after,
+                    )))))
+                }
+            }
             Self::Ref(tc_ref) => {
                 let tc_ref = tc_ref.reference_self(path);
                 Self::Ref(Box::new(tc_ref))
@@ -515,6 +521,26 @@ impl Refer for Scalar {
     }
 }
 
+impl<D: Digest> Hash<D> for Scalar {
+    fn hash(self) -> Output<D> {
+        Hash::<D>::hash(&self)
+    }
+}
+
+impl<'a, D: Digest> Hash<D> for &'a Scalar {
+    fn hash(self) -> Output<D> {
+        match self {
+            Scalar::Cluster(cluster_ref) => Hash::<D>::hash(cluster_ref),
+            Scalar::Map(map) => Hash::<D>::hash(map.deref()),
+            Scalar::Op(op) => Hash::<D>::hash(op),
+            Scalar::Range(range) => Hash::<D>::hash(range),
+            Scalar::Ref(tc_ref) => Hash::<D>::hash(tc_ref.deref()),
+            Scalar::Tuple(tuple) => Hash::<D>::hash(tuple.deref()),
+            Scalar::Value(value) => Hash::<D>::hash(value),
+        }
+    }
+}
+
 impl From<IdRef> for Scalar {
     fn from(id_ref: IdRef) -> Self {
         Self::Ref(Box::new(TCRef::Id(id_ref)))
@@ -766,6 +792,16 @@ impl TryCastFrom<Scalar> for Link {
     }
 }
 
+impl TryCastFrom<Scalar> for Float {
+    fn can_cast_from(scalar: &Scalar) -> bool {
+        Number::can_cast_from(scalar)
+    }
+
+    fn opt_cast_from(scalar: Scalar) -> Option<Self> {
+        Number::opt_cast_from(scalar).map(|n| n.cast_into())
+    }
+}
+
 impl TryCastFrom<Scalar> for Number {
     fn can_cast_from(scalar: &Scalar) -> bool {
         match scalar {
