@@ -1,5 +1,6 @@
 //! Resolve a reference conditionally.
 
+use async_hash::Hash;
 use std::collections::HashSet;
 use std::fmt;
 
@@ -7,6 +8,7 @@ use async_trait::async_trait;
 use destream::{de, en};
 use log::debug;
 use safecast::{Match, TryCastFrom, TryCastInto};
+use sha2::digest::{Digest, Output};
 
 use tc_error::*;
 use tcgeneric::{Id, Instance, PathSegment, TCPathBuf};
@@ -90,6 +92,12 @@ impl Refer for IfRef {
     }
 }
 
+impl<'a, D: Digest> Hash<D> for &'a IfRef {
+    fn hash(self) -> Output<D> {
+        Hash::<D>::hash((&self.cond, &self.then, &self.or_else))
+    }
+}
+
 impl TryCastFrom<Scalar> for IfRef {
     fn can_cast_from(scalar: &Scalar) -> bool {
         scalar.matches::<(TCRef, Scalar, Scalar)>()
