@@ -1,5 +1,6 @@
 //! Resolve a reference to an op.
 
+use async_hash::Hash;
 use std::collections::HashSet;
 use std::convert::{TryFrom, TryInto};
 use std::fmt;
@@ -12,13 +13,14 @@ use destream::en::{EncodeMap, Encoder, IntoStream, ToStream};
 use futures::{try_join, TryFutureExt};
 use log::debug;
 use safecast::{CastFrom, CastInto, TryCastFrom, TryCastInto};
+use sha2::digest::{Digest, Output};
 
 use tc_error::*;
 use tcgeneric::*;
 
 use crate::route::Public;
 use crate::scalar::{Link, Scalar, Scope, Value, SELF};
-use crate::state::{State, ToState};
+use crate::state::{State, StateType, ToState};
 use crate::txn::Txn;
 
 use super::{IdRef, Refer, TCRef};
@@ -82,6 +84,13 @@ pub enum Subject {
 }
 
 impl Subject {
+    pub fn as_class(&self) -> Option<StateType> {
+        match self {
+            Self::Link(link) => StateType::from_path(link.path()),
+            _ => None,
+        }
+    }
+
     fn dereference_self(self, path: &TCPathBuf) -> Self {
         match self {
             Self::Ref(id_ref, suffix) if id_ref.id() == &SELF => {
@@ -110,6 +119,15 @@ impl Subject {
     }
 }
 
+impl<'a, D: Digest> Hash<D> for &'a Subject {
+    fn hash(self) -> Output<D> {
+        match self {
+            Subject::Link(link) => Hash::<D>::hash(link),
+            Subject::Ref(id, path) => Hash::<D>::hash((id, path)),
+        }
+    }
+}
+
 impl From<IdRef> for Subject {
     fn from(id_ref: IdRef) -> Self {
         Self::Ref(id_ref, TCPathBuf::default())
@@ -426,32 +444,30 @@ impl Refer for OpRef {
         match self {
             Self::Get((subject, key)) => match subject {
                 Subject::Link(link) => {
-                    let key = key.resolve(context, txn).await?;
+                    let key = resolve(key, context, txn).await?;
                     let key = key.try_cast_into(invalid_key(&link))?;
-
                     txn.get(link, key).await
                 }
                 Subject::Ref(id_ref, path) => {
-                    let key = key.resolve(context, txn).await?;
+                    let key = resolve(key, context, txn).await?;
                     let key = key.try_cast_into(invalid_key(&id_ref))?;
-
                     context.resolve_get(txn, id_ref.id(), &path, key).await
                 }
             },
             Self::Put((subject, key, value)) => match subject {
                 Subject::Link(link) => {
-                    let key = key.resolve(context, txn).await?;
+                    let key = resolve(key, context, txn).await?;
                     let key = key.try_cast_into(invalid_key(&link))?;
 
-                    let value = value.resolve(context, txn).await?;
+                    let value = resolve(value, context, txn).await?;
 
                     txn.put(link, key, value)
                         .map_ok(|()| State::default())
                         .await
                 }
                 Subject::Ref(id_ref, path) => {
-                    let key = key.resolve(context, txn);
-                    let value = value.resolve(context, txn);
+                    let key = resolve(key, context, txn);
+                    let value = resolve(value, context, txn);
                     let (key, value) = try_join!(key, value)?;
 
                     let key = key.try_cast_into(invalid_key(&id_ref))?;
@@ -464,25 +480,23 @@ impl Refer for OpRef {
             },
             Self::Post((subject, params)) => match subject {
                 Subject::Link(link) => {
-                    let params = Scalar::Map(params).resolve(context, txn).await?;
+                    let params = resolve(Scalar::Map(params), context, txn).await?;
                     txn.post(link, params).await
                 }
                 Subject::Ref(id_ref, path) => {
-                    let params = Scalar::Map(params).resolve(context, txn).await?;
+                    let params = resolve(Scalar::Map(params), context, txn).await?;
                     let params = params.try_into()?;
-
                     context.resolve_post(txn, id_ref.id(), &path, params).await
                 }
             },
             Self::Delete((subject, key)) => match subject {
                 Subject::Link(link) => {
-                    let key = key.resolve(context, txn).await?;
+                    let key = resolve(key, context, txn).await?;
                     let key = key.try_cast_into(invalid_key(&link))?;
-
                     txn.delete(link, key).map_ok(|()| State::default()).await
                 }
                 Subject::Ref(id_ref, path) => {
-                    let key = key.resolve(context, txn).await?;
+                    let key = resolve(key, context, txn).await?;
                     let key = key.try_cast_into(invalid_key(&id_ref))?;
 
                     context
@@ -495,6 +509,17 @@ impl Refer for OpRef {
     }
 }
 
+impl<'a, D: Digest> Hash<D> for &'a OpRef {
+    fn hash(self) -> Output<D> {
+        match self {
+            OpRef::Get(get) => Hash::<D>::hash(get),
+            OpRef::Put(put) => Hash::<D>::hash(put),
+            OpRef::Post((subject, params)) => Hash::<D>::hash((subject, params.deref())),
+            OpRef::Delete(delete) => Hash::<D>::hash(delete),
+        }
+    }
+}
+
 impl CastFrom<OpRef> for Tuple<Scalar> {
     fn cast_from(value: OpRef) -> Self {
         match value {
@@ -533,6 +558,24 @@ impl OpRefVisitor {
                 Ok(OpRef::Get((subject, key)))
             }
             Scalar::Tuple(mut tuple) if tuple.len() == 2 => {
+                if let Some(class) = subject.as_class() {
+                    const ERR_IMMUTABLE: &str =
+                        "a scalar is immutable and cannot contain a mutable type";
+                    const HINT: &str = "(consider using a closure)";
+
+                    if let StateType::Chain(ct) = class {
+                        return Err(E::custom(format!(
+                            "{} {}: {} {}",
+                            ERR_IMMUTABLE, ct, tuple, HINT
+                        )));
+                    } else if let StateType::Collection(ct) = class {
+                        return Err(E::custom(format!(
+                            "{} {}: {} {}",
+                            ERR_IMMUTABLE, ct, tuple, HINT
+                        )));
+                    }
+                }
+
                 let value = tuple.pop().unwrap();
                 let key = tuple.pop().unwrap();
                 Ok(OpRef::Put((subject, key, value)))
@@ -640,3 +683,16 @@ impl fmt::Display for OpRef {
         }
     }
 }
+
+async fn resolve<'a, T, S>(tc_ref: T, context: &'a Scope<'a, S>, txn: &'a Txn) -> TCResult<State>
+where
+    T: Refer,
+    S: ToState + Public + Instance,
+{
+    let mut state = tc_ref.resolve(context, txn).await?;
+    while state.is_ref() {
+        state = state.resolve(context, txn).await?;
+    }
+
+    Ok(state)
+}
