@@ -5,20 +5,23 @@ use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::str::FromStr;
 
+use async_hash::Hash;
 use async_trait::async_trait;
 use bytes::Bytes;
 use destream::de;
 use futures::future::TryFutureExt;
-use futures::stream::{StreamExt, TryStreamExt};
+use futures::stream::{self, StreamExt, TryStreamExt};
 use log::debug;
 use safecast::{CastFrom, CastInto, TryCastFrom};
+use sha2::digest::{Digest, Output};
+use sha2::Sha256;
 
 use tc_error::*;
 use tc_transact::Transaction;
-use tc_value::{Link, Number, TCString, Value, ValueType};
+use tc_value::{Float, Link, Number, NumberType, TCString, Value, ValueType};
 use tcgeneric::*;
 
-use crate::chain::{self, Chain, ChainType, ChainVisitor};
+use crate::chain::{self, Chain, ChainInstance, ChainType, ChainVisitor};
 use crate::closure::*;
 use crate::collection::*;
 use crate::object::{InstanceClass, Object, ObjectType, ObjectVisitor};
@@ -31,6 +34,7 @@ pub use view::StateView;
 
 mod view;
 
+/// Trait defining a `State` representation of a (possibly non-`State`) value.
 pub trait ToState {
     fn to_state(&self) -> State;
 }
@@ -68,6 +72,7 @@ impl NativeClass for StateType {
         } else if &path[0] == "state" {
             if path.len() == 2 {
                 match path[1].as_str() {
+                    "closure" => Some(Self::Closure),
                     "map" => Some(Self::Map),
                     "tuple" => Some(Self::Tuple),
                     _ => None,
@@ -114,6 +119,12 @@ impl From<CollectionType> for StateType {
     }
 }
 
+impl From<NumberType> for StateType {
+    fn from(nt: NumberType) -> Self {
+        ValueType::from(nt).into()
+    }
+}
+
 impl From<ChainType> for StateType {
     fn from(ct: ChainType) -> Self {
         Self::Chain(ct)
@@ -182,6 +193,52 @@ pub enum State {
 }
 
 impl State {
+    /// Compute the SHA256 hash of this `State`.
+    pub fn hash<'a>(self, txn: Txn) -> TCBoxTryFuture<'a, Output<Sha256>> {
+        Box::pin(async move {
+            match self {
+                Self::Collection(collection) => collection.hash(txn).await,
+                Self::Chain(chain) => chain.hash(txn).await,
+                Self::Closure(closure) => closure.hash(txn).await,
+                Self::Map(map) => {
+                    let mut hashes = stream::iter(map)
+                        .map(|(id, state)| {
+                            state
+                                .hash(txn.clone())
+                                .map_ok(|hash| (Hash::<Sha256>::hash(id), hash))
+                        })
+                        .buffered(num_cpus::get())
+                        .map_ok(|(id, state)| {
+                            let mut inner_hasher = Sha256::default();
+                            inner_hasher.update(&id);
+                            inner_hasher.update(&state);
+                            inner_hasher.finalize()
+                        });
+
+                    let mut hasher = Sha256::default();
+                    while let Some(hash) = hashes.try_next().await? {
+                        hasher.update(&hash);
+                    }
+                    Ok(hasher.finalize())
+                }
+                Self::Object(object) => object.hash(txn).await,
+                Self::Scalar(scalar) => Ok(Hash::<Sha256>::hash(scalar)),
+                Self::Stream(stream) => stream.hash(txn).await,
+                Self::Tuple(tuple) => {
+                    let mut hashes = stream::iter(tuple)
+                        .map(|state| state.hash(txn.clone()))
+                        .buffered(num_cpus::get());
+
+                    let mut hasher = Sha256::default();
+                    while let Some(hash) = hashes.try_next().await? {
+                        hasher.update(&hash);
+                    }
+                    Ok(hasher.finalize())
+                }
+            }
+        })
+    }
+
     /// Return true if this `State` is an empty [`Tuple`] or [`Map`], default [`Link`], or `Value::None`
     pub fn is_none(&self) -> bool {
         match self {
@@ -456,6 +513,16 @@ impl From<Map<Scalar>> for State {
     }
 }
 
+impl From<Map<chain::Subject>> for State {
+    fn from(map: Map<chain::Subject>) -> Self {
+        Self::Map(
+            map.into_iter()
+                .map(|(name, subject)| (name, Self::from(subject)))
+                .collect(),
+        )
+    }
+}
+
 impl From<Number> for State {
     fn from(n: Number) -> Self {
         Self::Scalar(n.into())
@@ -635,6 +702,12 @@ impl TryFrom<State> for Map<State> {
     fn try_from(state: State) -> TCResult<Map<State>> {
         match state {
             State::Map(map) => Ok(map),
+
+            State::Scalar(Scalar::Map(map)) => Ok(map
+                .into_iter()
+                .map(|(id, scalar)| (id, State::Scalar(scalar)))
+                .collect()),
+
             other => Err(TCError::bad_request("expected Map but found", other)),
         }
     }
@@ -1029,6 +1102,22 @@ impl TryCastFrom<State> for Value {
     }
 }
 
+impl TryCastFrom<State> for Float {
+    fn can_cast_from(state: &State) -> bool {
+        match state {
+            State::Scalar(scalar) => Self::can_cast_from(scalar),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(state: State) -> Option<Self> {
+        match state {
+            State::Scalar(scalar) => Self::opt_cast_from(scalar),
+            _ => None,
+        }
+    }
+}
+
 impl TryCastFrom<State> for Number {
     fn can_cast_from(state: &State) -> bool {
         match state {
@@ -1045,6 +1134,22 @@ impl TryCastFrom<State> for Number {
     }
 }
 
+impl TryCastFrom<State> for u64 {
+    fn can_cast_from(state: &State) -> bool {
+        match state {
+            State::Scalar(scalar) => Self::can_cast_from(scalar),
+            _ => false,
+        }
+    }
+
+    fn opt_cast_from(state: State) -> Option<Self> {
+        match state {
+            State::Scalar(scalar) => Self::opt_cast_from(scalar),
+            _ => None,
+        }
+    }
+}
+
 impl fmt::Debug for State {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
@@ -1086,6 +1191,8 @@ impl StateVisitor {
         class: StateType,
         access: &mut A,
     ) -> Result<State, A::Error> {
+        debug!("decode instance of {}", class);
+
         match class {
             StateType::Chain(ct) => {
                 ChainVisitor::new(self.txn.clone())
